'''
    Copyright 2009, 2010 Oskar Andreasson

    This file is part of Screenmovie.

    Screenmovie is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Screenmovie is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Screenmovie.  If not, see <http://www.gnu.org/licenses/>.

@author: oan
'''
import pygtk
pygtk.require('2.0')
import gtk
import gio
import pickle
import os

class Preferences():
    '''
    classdocs
    '''
    prefsFile = os.path.expanduser("~") + "/.Screenmovierc"

    saveDirectory = os.path.expanduser("~") + "/Video"
    fileFormat = "avi"

    videoEnabled = True
    frameRate = 15
    codec = "huffyuv"

    soundEnabled = False
    soundSystem = "alsa"
    soundInput = "plughw:0,0"
    soundBitrate = 128000
    soundSampleFrequency = 48000
    soundOutputCodec = "vorbis"
    
    postprocessingEnabled = False
    postprocessingFileFormat = "avi"
    postprocessingVideoCodec = "h264"
    postprocessingSoundCodec = "ac3"
    
    def _loadPreferences(self):
        print "Load preferences"
        try:
            file = open(self.prefsFile, "r")
            unpickled = pickle.load(file)
            file.close()
            self.frameRate = unpickled[0]
            self.saveDirectory = unpickled[1]
            self.codec = unpickled[2]
            self.soundSystem = unpickled[3]
            self.soundInput = unpickled[4]
            self.soundBitrate = unpickled[5]
            self.soundSampleFrequency = unpickled[6]
            self.soundOutputCodec = unpickled[7]
            self.videoEnabled = unpickled[8]
            self.soundEnabled = unpickled[9]
            self.postprocessingEnabled = unpickled[10]
            self.fileFormat = unpickled[11] 
            pass
        except:
            print "Failed loading preferences!"
            pass
    
    def _savePreferences(self):
        print "Save preferences"
        try:
            pickles = [self.frameRate, 
                       self.saveDirectory, 
                       self.codec, 
                       self.soundSystem, 
                       self.soundInput, 
                       self.soundBitrate,
                       self.soundSampleFrequency,
                       self.soundOutputCodec,
                       self.videoEnabled,
                       self.soundEnabled,
                       self.postprocessingEnabled,
                       self.fileFormat]
            file = open(self.prefsFile, "w")
            pickle.dump(pickles, file)
            file.close()
        except: 
            print "Failed saving preferences!"
            pass

    def _videoEnabledCheckBoxToggled(self, data):
        if self.videoEnabledCheckButton.get_active():
            self.frameRateSpinbox.set_sensitive(True)
            self.codecComboBox.set_sensitive(True)
        else:
            self.frameRateSpinbox.set_sensitive(False)
            self.codecComboBox.set_sensitive(False)
        
    def _showVideoSettings(self):
        vbox = gtk.VBox()
        
        # Enable/Disable video 
        videoEnableBox = gtk.HBox()
        vbox.pack_start(videoEnableBox, expand=False, fill=True, padding=2)
        videoEnableLabel = gtk.Label("Enable video recording:")
        videoEnableBox.pack_start(videoEnableLabel, expand=False, fill=True)
        self.videoEnabledCheckButton = gtk.CheckButton()
        videoEnableBox.pack_start(self.videoEnabledCheckButton, expand=False, fill=True)
        self.videoEnabledCheckButton.connect("toggled", self._videoEnabledCheckBoxToggled)
       
        #frameRate
        frameRateBox = gtk.HBox()
        vbox.pack_start(frameRateBox, expand=False, fill=True, padding=10)
        frameRateLabel = gtk.Label("Frame Rate:")
        frameRateLabel.set_justify(gtk.JUSTIFY_LEFT)
        frameRateBox.pack_start(frameRateLabel, expand=False, fill=True)
        frameRateAdjustment = gtk.Adjustment(value=self.frameRate, lower=1, upper=40, step_incr=1, page_incr=5)
        self.frameRateSpinbox = gtk.SpinButton(adjustment=frameRateAdjustment)
        frameRateBox.pack_end(self.frameRateSpinbox, expand=False, fill=True)
        
        # Codec choice
        codecBox = gtk.HBox()
        vbox.pack_start(codecBox, expand=False, fill=True, padding=10)
        codecLabel = gtk.Label("Codec (still unused):")
        codecBox.pack_start(codecLabel, expand=False, fill=True)
        self.codecComboBox = gtk.combo_box_entry_new_text()
        self.codecComboBox.append_text(self.codec)
        self.codecComboBox.append_text("huffyuv")
        self.codecComboBox.append_text("cyuv")
        self.codecComboBox.append_text("h264")
        self.codecComboBox.append_text("mpeg2")
        self.codecComboBox.append_text("mpeg4")
        self.codecComboBox.append_text("pgmyuv")
        self.codecComboBox.append_text("theora")
        self.codecComboBox.set_active(0)
        codecBox.pack_end(self.codecComboBox, expand=False, fill=True)

        # Has to be set last so everything is set up when callback is run
        self.videoEnabledCheckButton.set_active(self.videoEnabled)

        return vbox

    def _showGeneralSettings(self):
        vbox = gtk.VBox()

        # Save directory
        fileBox = gtk.HBox()
        vbox.pack_start(fileBox, expand=False, fill=True, padding=10)
        fileLabel = gtk.Label("Save location:")
        fileLabel.set_justify(gtk.JUSTIFY_LEFT)
        fileBox.pack_start(fileLabel, expand=False, fill=False)
        self.fileChooser = gtk.FileChooserButton("Save directory")
        self.fileChooser.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        try:
            self.fileChooser.set_file(gio.File(self.saveDirectory))
        except:
            self.fileChooser.unselect_all()
            self.fileChooser.select_filename(self.saveDirectory)
        fileBox.pack_end(self.fileChooser, expand=False, fill=True)
        
        # File formats
        # self.fileFormat = "alsa"
        fileFormatBox = gtk.HBox()
        vbox.pack_start(fileFormatBox, expand=False, fill=True, padding=2)
        fileFormatLabel = gtk.Label("File format:")
        fileFormatBox.pack_start(fileFormatLabel, expand=False, fill=True)
        self.fileFormatCombo = gtk.combo_box_entry_new_text()
        self.fileFormatCombo.append_text(self.fileFormat)
        self.fileFormatCombo.append_text("avi")
        self.fileFormatCombo.append_text("matroska")
        self.fileFormatCombo.set_active(0)
        fileFormatBox.pack_end(self.fileFormatCombo, expand=False, fill=True)

        
        return vbox
    
    def _soundEnabledCheckButtonToggled(self, data):
        if self.soundEnabledCheckButton.get_active():
            self.soundBitrateSpinbox.set_sensitive(True)
            self.soundInputCombo.set_sensitive(True)
            self.soundSystemCombo.set_sensitive(True)
            self.soundOutputCodecCombo.set_sensitive(True)
            self.soundSampleFrequencySpinbox.set_sensitive(True)
        else:
            self.soundBitrateSpinbox.set_sensitive(False)
            self.soundInputCombo.set_sensitive(False)
            self.soundSystemCombo.set_sensitive(False)
            self.soundOutputCodecCombo.set_sensitive(False)
            self.soundSampleFrequencySpinbox.set_sensitive(False)

    def _showAudioSettings(self):
        vbox = gtk.VBox()
        
        # Enable/Disable sound 
        soundEnableBox = gtk.HBox()
        vbox.pack_start(soundEnableBox, expand=False, fill=True, padding=2)
        soundEnableLabel = gtk.Label("Enable sound recording:")
        soundEnableBox.pack_start(soundEnableLabel, expand=False, fill=True)
        self.soundEnabledCheckButton = gtk.CheckButton()
        soundEnableBox.pack_start(self.soundEnabledCheckButton, expand=False, fill=True)
        self.soundEnabledCheckButton.connect("toggled", self._soundEnabledCheckButtonToggled)

        # Codec choice
        # self.soundSystem = "alsa"
        soundSystemBox = gtk.HBox()
        vbox.pack_start(soundSystemBox, expand=False, fill=True, padding=2)
        soundSystemLabel = gtk.Label("Sound System:")
        soundSystemBox.pack_start(soundSystemLabel, expand=False, fill=True)
        self.soundSystemCombo = gtk.combo_box_entry_new_text()
        self.soundSystemCombo.append_text(self.soundSystem)
        self.soundSystemCombo.append_text("alsa")
        self.soundSystemCombo.set_active(0)
        soundSystemBox.pack_end(self.soundSystemCombo, expand=False, fill=True)

        # Sound input device
        # soundInput = "plughw:0,0"
        # TODO: Find real alsa inputs
        soundInputBox = gtk.HBox()
        vbox.pack_start(soundInputBox, expand=False, fill=True, padding=2)
        soundInputLabel = gtk.Label("Sound Input:")
        soundInputBox.pack_start(soundInputLabel, expand=False, fill=True)
        self.soundInputCombo = gtk.combo_box_entry_new_text()
        self.soundInputCombo.append_text(self.soundInput)
        self.soundInputCombo.append_text("pulse")
        self.soundInputCombo.append_text("plughw:0,0")
        self.soundInputCombo.append_text("plughw:0,1")
        self.soundInputCombo.append_text("plughw:1,0")
        self.soundInputCombo.append_text("plughw:1,1")        
        self.soundInputCombo.set_active(0)
        soundInputBox.pack_end(self.soundInputCombo, expand=False, fill=True)

        # Sound bitrate
        # soundBitrate = 128000
        soundBitrateBox = gtk.HBox()
        vbox.pack_start(soundBitrateBox, expand=False, fill=True, padding=2)
        soundBitrateLabel = gtk.Label("Bitrate:")
        soundBitrateLabel.set_justify(gtk.JUSTIFY_LEFT)
        soundBitrateBox.pack_start(soundBitrateLabel, expand=False, fill=True)
        soundBitrateAdjustment = gtk.Adjustment(value=self.soundBitrate, lower=9000, upper=512000, step_incr=1000, page_incr=10000)
        self.soundBitrateSpinbox = gtk.SpinButton(adjustment=soundBitrateAdjustment)
        soundBitrateBox.pack_end(self.soundBitrateSpinbox, expand=False, fill=True)

        # Sound Sample frequency
        # soundSampleFrequency = 48000
        soundSampleFrequencyBox = gtk.HBox()
        vbox.pack_start(soundSampleFrequencyBox, expand=False, fill=True, padding=2)
        soundSampleFrequencyLabel = gtk.Label("Sample frequency:")
        soundSampleFrequencyLabel.set_justify(gtk.JUSTIFY_LEFT)
        soundSampleFrequencyBox.pack_start(soundSampleFrequencyLabel, expand=False, fill=True)
        soundSampleFrequencyAdjustment = gtk.Adjustment(value=self.soundSampleFrequency, lower=1000, upper=100000, step_incr=1000, page_incr=10000)
        self.soundSampleFrequencySpinbox = gtk.SpinButton(adjustment=soundSampleFrequencyAdjustment)
        soundSampleFrequencyBox.pack_end(self.soundSampleFrequencySpinbox, expand=False, fill=True)
        
        # Sound Output codec
        # soundOutputCodec = "ac3"
        # TODO: Find real alsa inputs from aplay for example
        soundOutputCodecBox = gtk.HBox()
        vbox.pack_start(soundOutputCodecBox, expand=False, fill=True, padding=2)
        soundOutputCodecLabel = gtk.Label("Output codec:")
        soundOutputCodecBox.pack_start(soundOutputCodecLabel, expand=False, fill=True)
        self.soundOutputCodecCombo = gtk.combo_box_entry_new_text()
        self.soundOutputCodecCombo.append_text(self.soundOutputCodec)
        self.soundOutputCodecCombo.append_text("ac3")
        self.soundOutputCodecCombo.append_text("flac")
        self.soundOutputCodecCombo.append_text("adpcm_ms")
        self.soundOutputCodecCombo.append_text("mp2")        
        self.soundOutputCodecCombo.append_text("vorbis")        
        self.soundOutputCodecCombo.set_active(0)
        soundOutputCodecBox.pack_end(self.soundOutputCodecCombo, expand=False, fill=True)

        # Has to be set last so all things get set etc
        self.soundEnabledCheckButton.set_active(self.soundEnabled)

        return vbox
    
    def _postprocessingEnabledCheckButtonToggled(self, data):
        if self.postprocessingEnabledCheckButton.get_active():
            pass
        else:
            pass

    def _showPostprocessingSettings(self):
        vbox = gtk.VBox()
        
        # Enable/Disable sound 
        postprocessingEnableBox = gtk.HBox()
        vbox.pack_start(postprocessingEnableBox, expand=False, fill=True, padding=2)
        postprocessingEnableLabel = gtk.Label("Enable sound recording:")
        postprocessingEnableBox.pack_start(postprocessingEnableLabel, expand=False, fill=True)
        self.postprocessingEnabledCheckButton = gtk.CheckButton()
        postprocessingEnableBox.pack_start(self.postprocessingEnabledCheckButton, expand=False, fill=True)
        self.postprocessingEnabledCheckButton.connect("toggled", self._postprocessingEnabledCheckButtonToggled)


        
        # Has to be set last so all things get set etc
        self.postprocessingEnabledCheckButton.set_active(self.postprocessingEnabled)
        
        return vbox

    def showGui(self):
        self.prefWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.prefWindow.set_title("Screenmovie Preferences")
        self.prefWindow.set_border_width(2)
        vbox = gtk.VBox()

        notebook = gtk.Notebook()
        vbox.pack_start(notebook, expand=True, fill=True, padding=0)

        generalSettings = self._showGeneralSettings()
        notebook.append_page(generalSettings, gtk.Label("General"))
        videoSettings = self._showVideoSettings()
        notebook.append_page(videoSettings, gtk.Label("Video"))
        audioSettings = self._showAudioSettings()
        notebook.append_page(audioSettings, gtk.Label("Audio"))
        postprocessingSettings = self._showPostprocessingSettings()
        notebook.append_page(postprocessingSettings, gtk.Label("Postprocessing"))

        self.prefWindow.add(vbox)
        
        saveAndApplyBox = gtk.HBox()
        vbox.pack_start(saveAndApplyBox, expand=False, fill=True, padding=1)
        okButton = gtk.Button("Ok")
        saveAndApplyBox.pack_end(okButton, expand=False, fill=True, padding=0)
        cancelButton = gtk.Button("Cancel")
        saveAndApplyBox.pack_end(cancelButton, expand=False, fill=True, padding=0)

        cancelButton.connect_object("clicked", self.applyPressed, self.prefWindow)
        okButton.connect_object("clicked", self.okPressed, self.prefWindow)
        self.prefWindow.connect("delete-event", self.deletePressed)

        self.prefWindow.show_all()
        pass
    
    def applyPressed(self, event):
        gtk.Widget.destroy(event)
        pass
    
    def okPressed(self, event):
        self.frameRate = self.frameRateSpinbox.get_value_as_int()
        self.codec = self.codecComboBox.get_active_text()
        self.saveDirectory = self.fileChooser.get_filename()

        self.soundSystem = self.soundSystemCombo.get_active_text()
        self.soundInput = self.soundInputCombo.get_active_text()
        self.soundBitrate = self.soundBitrateSpinbox.get_value_as_int()
        self.soundSampleFrequency = self.soundSampleFrequencySpinbox.get_value_as_int()
        self.soundOutputCodec = self.soundOutputCodecCombo.get_active_text()
        
        self.fileFormat = self.fileFormatCombo.get_active_text()
        
        self.videoEnabled = self.videoEnabledCheckButton.get_active()
        self.soundEnabled = self.soundEnabledCheckButton.get_active()
        self.postprocessingEnabled = self.postprocessingEnabledCheckButton.get_active()

        self._savePreferences()
        gtk.Widget.destroy(event)
        pass
    
    def deletePressed(self, event, lala):
        gtk.Widget.destroy(event)
        exit()

    def __init__(self):
        '''
        Constructor
        '''
        self._loadPreferences()
        
    def __del__(self):
        '''
        Destructor
        '''
        self._savePreferences()
