#!/usr/bin/python
'''
    Copyright 2009, 2010 Oskar Andreasson

    This file is part of Screenmovie.

    Screenmovie is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Screenmovie is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Screenmovie.  If not, see <http://www.gnu.org/licenses/>.

@author: oan
'''
import pygtk
pygtk.require('2.0')
import gtk
import subprocess
from datetime import datetime
import Preferences

class RecordTray:

    def __init__(self):
        # Start state is stopped
        self.statusIcon = gtk.StatusIcon()
        self.statusIcon.set_from_stock(gtk.STOCK_MEDIA_STOP)
        self.statusIcon.set_visible(True)
        self.statusIcon.set_tooltip("Stopped")
        self.activated = 0
    
        self.menu = gtk.Menu()
        self.menuItem = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES)
        self.menuItem.connect('activate', self.preferences, self.statusIcon)
        self.menu.append(self.menuItem)
        self.menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
        self.menuItem.connect('activate', self.quit_cb, self.statusIcon)
        self.menu.append(self.menuItem)
    
        self.statusIcon.connect('popup-menu', self.popup_menu_cb, self.menu)
        self.activate_stopped(None)
        self.statusIcon.set_visible(1)
        self.prefs = Preferences.Preferences()
        gtk.main()
        
    def checkValue(self, line):
        '''
        Helper function for activate_paused to check the values of each line output by the xwininfo application for 
        interesting information, and set left, top, width and height attributes for recording
        '''
        if "Absolute upper-left X:" in line:
            self.left = int(line.split()[3])
        elif "Absolute upper-left Y:" in line:
            self.top = line.split()[3]
        elif "Width:" in line:
            if int(line.split()[1])%2:
                self.width = int(line.split()[1])+1
            else:
                self.width = line.split()[1]
        elif "Height:" in line:
            if int(line.split()[1])%2:
                self.height = int(line.split()[1])+1
            else:
                self.height = line.split()[1]
        pass
    
    def activate_paused(self, data):
        # Awaiting mouse click for window information 
        self.statusIcon.set_from_stock(gtk.STOCK_MEDIA_RECORD)
        self.statusIcon.set_tooltip("Recording")
        self.pipe = subprocess.Popen([r"xwininfo"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        self.pipe.wait()
        while self.pipe.returncode != 0:
            self.pipe = subprocess.Popen([r"xwininfo"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            self.pipe.wait()
        
        for line in self.pipe.stdout:
            self.checkValue(line)

        runstring = ["ffmpeg"]
        if self.prefs.soundEnabled:
            runstring = runstring + ["-f", str(self.prefs.soundSystem),
                     "-i", str(self.prefs.soundInput), 
                     "-ab", str(self.prefs.soundBitrate),
                     "-ar", str(self.prefs.soundSampleFrequency),
                     "-ac", str(1),
                     "-acodec", str(self.prefs.soundOutputCodec)]
        if self.prefs.soundEnabled and self.prefs.videoEnabled:
            runstring = runstring + ["-isync"]
        if self.prefs.videoEnabled:
            runstring = runstring + ["-f", "x11grab", 
                     "-r", str(self.prefs.frameRate), 
                     "-s", str(self.width) + "x" + str(self.height), 
                     "-i", ":0.0+" + str(self.left) + "," + str(self.top), 
                     "-vcodec", str(self.prefs.codec), 
                     "-sameq"]
        self.recordingFileName = str(self.prefs.saveDirectory) + "/Screenmovie-" + str(datetime.now().strftime("%Y%m%d-%H%M%S")) + "." + self.prefs.fileFormat
        runstring = runstring + [self.recordingFileName]
        print runstring
        self.pipe = subprocess.Popen(runstring)

        if self.activated:
            self.statusIcon.disconnect(self.activated)
        self.activated = self.statusIcon.connect('activate', self.activate_stopped)
        
    def activate_stopped(self, data):
        # The user clicked the icon again, stop the recording
        '''
            postprocessingEnabled = False
            postprocessingFileFormat = "avi"
            postprocessingVideoCodec = "h264"
            postprocessingSoundCodec = "ac3"
        '''
        if hasattr(self, "pipe"):
            self.pipe.terminate()
        #TODO Add ifisfile and postprocessing then recode to above formats
        self.statusIcon.set_from_stock(gtk.STOCK_MEDIA_STOP)
        self.statusIcon.set_tooltip("Stopped")
        if self.activated:
            self.statusIcon.disconnect(self.activated)
        self.activated = self.statusIcon.connect('activate', self.activate_paused)

    def preferences(self, widget, event, data = None):
        self.prefs.showGui()
        
    def quit_cb(self, widget, data = None):
        gtk.main_quit()

    def popup_menu_cb(self, widget, button, time, data = None):
        if button == 3:
            if data:
                data.show_all()
                data.popup(None, None, gtk.status_icon_position_menu,
                       3, time, self.statusIcon)

if __name__ == "__main__":
    recordTray = RecordTray()
